@@ -692,7 +692,8 @@ def _StartupInstance(name, opts):
   """
   op = opcodes.OpInstanceStartup(instance_name=name,
                                  force=opts.force,
-                                 ignore_offline_nodes=opts.ignore_offline)
+                                 ignore_offline_nodes=opts.ignore_offline,
+                                 no_remember=opts.no_remember)
   # do not add these parameters to the opcode unless they're defined
   if opts.hvparams:
     op.hvparams = opts.hvparams
@@ -731,7 +732,8 @@ def _ShutdownInstance(name, opts):
   """
   return opcodes.OpInstanceShutdown(instance_name=name,
                                     timeout=opts.timeout,
-                                    ignore_offline_nodes=opts.ignore_offline)
+                                    ignore_offline_nodes=opts.ignore_offline,
+                                    no_remember=opts.no_remember)
 
 
 def ReplaceDisks(opts, args):
@@ -1457,14 +1459,15 @@ commands = {
     [m_node_opt, m_pri_node_opt, m_sec_node_opt, m_clust_opt,
      m_node_tags_opt, m_pri_node_tags_opt, m_sec_node_tags_opt,
      m_inst_tags_opt, m_inst_opt, m_force_multi, TIMEOUT_OPT, SUBMIT_OPT,
-     DRY_RUN_OPT, PRIORITY_OPT, IGNORE_OFFLINE_OPT],
+     DRY_RUN_OPT, PRIORITY_OPT, IGNORE_OFFLINE_OPT, NO_REMEMBER_OPT],
     "<instance>", "Stops an instance"),
   'startup': (
     GenericManyOps("startup", _StartupInstance), [ArgInstance()],
     [FORCE_OPT, m_force_multi, m_node_opt, m_pri_node_opt, m_sec_node_opt,
      m_node_tags_opt, m_pri_node_tags_opt, m_sec_node_tags_opt,
      m_inst_tags_opt, m_clust_opt, m_inst_opt, SUBMIT_OPT, HVOPTS_OPT,
-     BACKEND_OPT, DRY_RUN_OPT, PRIORITY_OPT, IGNORE_OFFLINE_OPT],
+     BACKEND_OPT, DRY_RUN_OPT, PRIORITY_OPT, IGNORE_OFFLINE_OPT,
+     NO_REMEMBER_OPT],
     "<instance>", "Starts an instance"),
   'reboot': (
     GenericManyOps("reboot", _RebootInstance), [ArgInstance()],
