@@ -115,7 +115,7 @@ def _Tags_PUT(kind, tags, name, dry_run):
   """Helper function to set tags.
 
   """
-  return SubmitJob([opcodes.OpAddTags(kind=kind, name=name,
+  return SubmitJob([opcodes.OpTagsSet(kind=kind, name=name,
                                       tags=tags, dry_run=dry_run)])
 
 
@@ -123,7 +123,7 @@ def _Tags_DELETE(kind, tags, name, dry_run):
   """Helper function to delete tags.
 
   """
-  return SubmitJob([opcodes.OpDelTags(kind=kind, name=name,
+  return SubmitJob([opcodes.OpTagsDel(kind=kind, name=name,
                                       tags=tags, dry_run=dry_run)])
 
 
@@ -170,6 +170,46 @@ def MakeParamsDict(opts, params):
   return result
 
 
+def FillOpcode(opcls, body, static):
+  """Fills an opcode with body parameters.
+
+  Parameter types are checked.
+
+  @type opcls: L{opcodes.OpCode}
+  @param opcls: Opcode class
+  @type body: dict
+  @param body: Body parameters as received from client
+  @type static: dict
+  @param static: Static parameters which can't be modified by client
+  @return: Opcode object
+
+  """
+  CheckType(body, dict, "Body contents")
+
+  if static:
+    overwritten = set(body.keys()) & set(static.keys())
+    if overwritten:
+      raise http.HttpBadRequest("Can't overwrite static parameters %r" %
+                                overwritten)
+
+  # Combine parameters
+  params = body.copy()
+
+  if static:
+    params.update(static)
+
+  # Convert keys to strings (simplejson decodes them as unicode)
+  params = dict((str(key), value) for (key, value) in params.items())
+
+  try:
+    op = opcls(**params) # pylint: disable-msg=W0142
+    op.Validate(False)
+  except (errors.OpPrereqError, TypeError), err:
+    raise http.HttpBadRequest("Invalid body parameters: %s" % err)
+
+  return op
+
+
 def SubmitJob(op, cl=None):
   """Generic wrapper for submit job, for better http compatibility.
 
@@ -190,7 +230,10 @@ def SubmitJob(op, cl=None):
   except errors.JobQueueDrainError:
     raise http.HttpServiceUnavailable("Job queue is drained, cannot submit")
   except luxi.NoMasterError, err:
-    raise http.HttpBadGateway("Master seems to unreachable: %s" % str(err))
+    raise http.HttpBadGateway("Master seems to be unreachable: %s" % str(err))
+  except luxi.PermissionError:
+    raise http.HttpInternalServerError("Internal error: no permission to"
+                                       " connect to the master daemon")
   except luxi.TimeoutError, err:
     raise http.HttpGatewayTimeout("Timeout while talking to the master"
                                   " daemon. Error: %s" % str(err))
@@ -217,17 +260,21 @@ def GetClient():
     return luxi.Client()
   except luxi.NoMasterError, err:
     raise http.HttpBadGateway("Master seems to unreachable: %s" % str(err))
+  except luxi.PermissionError:
+    raise http.HttpInternalServerError("Internal error: no permission to"
+                                       " connect to the master daemon")
 
 
-def FeedbackFn(ts, log_type, log_msg): # pylint: disable-msg=W0613
-  """Feedback logging function for http case.
+def FeedbackFn(msg):
+  """Feedback logging function for jobs.
 
   We don't have a stdout for printing log messages, so log them to the
   http log at least.
 
-  @param ts: the timestamp (unused)
+  @param msg: the message
 
   """
+  (_, log_type, log_msg) = msg
   logging.info("%s: %s", log_type, log_msg)
 
 
@@ -297,14 +344,15 @@ class R_Generic(object):
     """
     self.items = items
     self.queryargs = queryargs
-    self.req = req
-    self.sn = None
+    self._req = req
 
-  def getSerialNumber(self):
-    """Get Serial Number.
+  def _GetRequestBody(self):
+    """Returns the body data.
 
     """
-    return self.sn
+    return self._req.private.body_data
+
+  request_body = property(fget=_GetRequestBody)
 
   def _checkIntVariable(self, name, default=0):
     """Return the parsed value of an int argument.
@@ -345,30 +393,30 @@ class R_Generic(object):
 
     """
     if args:
-      return CheckParameter(self.req.request_body, name, default=args[0])
+      return CheckParameter(self.request_body, name, default=args[0])
 
-    return CheckParameter(self.req.request_body, name)
+    return CheckParameter(self.request_body, name)
 
   def useLocking(self):
     """Check if the request specifies locking.
 
     """
-    return self._checkIntVariable('lock')
+    return bool(self._checkIntVariable("lock"))
 
   def useBulk(self):
     """Check if the request specifies bulk querying.
 
     """
-    return self._checkIntVariable('bulk')
+    return bool(self._checkIntVariable("bulk"))
 
   def useForce(self):
     """Check if the request specifies a forced operation.
 
     """
-    return self._checkIntVariable('force')
+    return bool(self._checkIntVariable("force"))
 
   def dryRun(self):
     """Check if the request specifies dry-run mode.
 
     """
-    return self._checkIntVariable('dry-run')
+    return bool(self._checkIntVariable("dry-run"))
