@@ -115,6 +115,16 @@ def NiceSort(values, key=None):
   return sorted(values, key=keyfunc)
 
 
+def InvertDict(dict_in):
+  """Inverts the key/value mapping of a dict.
+
+  @param dict_in: The dict to invert
+  @return: the inverted dict
+
+  """
+  return dict(zip(dict_in.values(), dict_in.keys()))
+
+
 class RunningTimeout(object):
   """Class to calculate remaining timeout when doing several operations.
 
