@@ -26,6 +26,7 @@ import unittest
 import re
 
 from ganeti import constants
+from ganeti import locking
 
 import testutils
 
@@ -68,6 +69,14 @@ class TestConstants(unittest.TestCase):
     self.failUnless(constants.SSL_CERT_EXPIRATION_ERROR <
                     constants.SSL_CERT_EXPIRATION_WARN)
 
+  def testOpCodePriority(self):
+    self.failUnless(constants.OP_PRIO_LOWEST > constants.OP_PRIO_LOW)
+    self.failUnless(constants.OP_PRIO_LOW > constants.OP_PRIO_NORMAL)
+    self.failUnlessEqual(constants.OP_PRIO_NORMAL, locking._DEFAULT_PRIORITY)
+    self.failUnlessEqual(constants.OP_PRIO_DEFAULT, locking._DEFAULT_PRIORITY)
+    self.failUnless(constants.OP_PRIO_NORMAL > constants.OP_PRIO_HIGH)
+    self.failUnless(constants.OP_PRIO_HIGH > constants.OP_PRIO_HIGHEST)
+
 
 class TestParameterNames(unittest.TestCase):
   """HV/BE parameter tests"""
@@ -77,6 +86,8 @@ class TestParameterNames(unittest.TestCase):
     for kind, source in [('hypervisor', constants.HVS_PARAMETER_TYPES),
                          ('backend', constants.BES_PARAMETER_TYPES),
                          ('nic', constants.NICS_PARAMETER_TYPES),
+                         ("instdisk", constants.IDISK_PARAMS_TYPES),
+                         ("instnic", constants.INIC_PARAMS_TYPES),
                         ]:
       for key in source:
         self.failUnless(self.VALID_NAME.match(key),
